import { describe, it, expect } from 'vitest';
import {
  addWorkdays,
  isDateTimeInInterval,
  correctTimeZone,
  calcWorkdaysWithHolidays,
  calcWorkdays,
  isWorkday,
  toLuxonDateTime
} from '../src/datetime.js';

// --- isDateTimeInInterval ---
describe('isDateTimeInInterval', () => {
  it('returns true for datetime within interval (same timezone)', () => {
    const start = '2023-11-01T00:00:00Z';
    const end = '2023-12-01T12:00:00Z';
    const date = '2023-12-01T06:00:00Z';
    expect(isDateTimeInInterval(date, start, end)).toBe(true);
  });

  it('returns false for datetime outside interval (same timezone)', () => {
    const start = '2023-11-01T00:00:00Z';
    const end = '2023-12-01T12:00:00Z';
    const date = '2023-12-01T12:15:00Z';
    expect(isDateTimeInInterval(date, start, end)).toBe(false);
  });

  it('returns true for datetime within interval (different timezone)', () => {
    const start = '2023-11-01T00:00:00Z';
    const end = '2023-12-01T12:00:00Z';
    const date = '2023-11-01T03:15:00+03:00';
    expect(isDateTimeInInterval(date, start, end)).toBe(true);
  });

  it('returns false for datetime outside interval (different timezone)', () => {
    const start = '2023-11-01T00:00:00Z';
    const end = '2023-12-01T00:00:00Z';
    const date = '2023-12-01T03:15:00+03:00';
    expect(isDateTimeInInterval(date, start, end)).toBe(false);
  });
});

// --- isWorkday ---
describe('isWorkday', () => {
  it('returns true for a regular workday', () => {
    const date = '2023-11-06T03:15:00+03:00';
    expect(isWorkday(date)).toBe(true);
  });

  it('returns true when date is in holiday object but function still considers it a workday', () => {
    const date = '2023-11-06T03:15:00+03:00';
    const holidays = {
      from: '2023-11-06T00:00:00+03:00',
      to: '2023-11-06T23:59:59+03:00'
    };
    expect(isWorkday(date, holidays)).toBe(true);
  });

  it('handles array of holidays correctly', () => {
    const date1 = '2023-11-06T01:00:00+03:00';
    const date2 = '2023-11-08T01:00:00+03:00';
    const holidays = [
      {
        from: '2023-01-01T00:00:00+03:00',
        to: '2023-01-10T23:59:59+03:00'
      },
      {
        from: '2023-11-06T00:00:00+03:00',
        to: '2023-11-06T23:59:59+03:00'
      }
    ];
    expect(isWorkday(date1, holidays)).toBe(false);
    expect(isWorkday(date2, holidays)).toBe(true);
  });
});

// --- addWorkdays ---
describe('addWorkdays', () => {
  it('adds workdays without holidays', () => {
    const date = '2023-11-06T03:15:00+03:00';
    const result = addWorkdays(date, 2);
    const goal = toLuxonDateTime('2023-11-08T23:59:59.999+03:00');
    expect(result).toStrictEqual(goal);
  });

  it('adds workdays with a holiday object', () => {
    const date = '2023-11-03T07:48:56.489+03:00';
    const holidays = {
      from: '2023-11-06T00:00:00.000Z',
      to: '2023-11-06T23:59:59.999Z'
    };
    const result = addWorkdays(date, 1, undefined, holidays);
    const goal = toLuxonDateTime('2023-11-07T23:59:59.999+03:00');
    expect(result.toISO()).toBe(goal.toISO());
  });

  it('adds workdays with an array of holidays', () => {
    const date = '2023-11-03T07:48:56.489+03:00';
    const holidays = [
      {
        from: '2023-11-06T00:00:00.000Z',
        to: '2023-11-06T23:59:59.999Z'
      }
    ];
    const result = addWorkdays(date, 1, undefined, holidays);
    const goal = toLuxonDateTime('2023-11-07T23:59:59.999+03:00');
    expect(result.toISO()).toBe(goal.toISO());
  });
});

// --- calcWorkdays ---
describe('calcWorkdays', () => {
  it('returns 5 workdays (Mon to Sun)', () => {
    const start = '2023-10-30T00:00:00.000+03:00';
    const end = '2023-11-05T23:59:59.000+03:00';
    expect(calcWorkdays(start, end)).toBe(5);
  });

  it('returns 5 workdays (Mon to Fri)', () => {
    const start = '2023-10-30T00:00:00.000+03:00';
    const end = '2023-11-03T23:59:59.000+03:00';
    expect(calcWorkdays(start, end)).toBe(5);
  });

  it('returns 9 workdays (Tue to Sun)', () => {
    const start = '2023-10-31T00:00:00.000+03:00';
    const end = '2023-11-12T23:59:59.000+03:00';
    expect(calcWorkdays(start, end)).toBe(9);
  });

  it('returns 4 workdays (Tue to Sun short)', () => {
    const start = '2023-10-31T00:00:00.000+03:00';
    const end = '2023-11-05T23:59:59.000+03:00';
    expect(calcWorkdays(start, end)).toBe(4);
  });

  it('returns 5 workdays (Mon to Sat)', () => {
    const start = '2023-10-30T00:00:00.000+03:00';
    const end = '2023-11-04T23:59:59.000+03:00';
    expect(calcWorkdays(start, end)).toBe(5);
  });

  it('returns 10 workdays (Mon to Sat long)', () => {
    const start = '2023-10-30T00:00:00.000+03:00';
    const end = '2023-11-11T23:59:59.000+03:00';
    expect(calcWorkdays(start, end)).toBe(10);
  });

  it('returns 10 workdays (Mon to Fri long)', () => {
    const start = '2023-10-30T00:00:00.000+03:00';
    const end = '2023-11-10T23:59:59.000+03:00';
    expect(calcWorkdays(start, end)).toBe(10);
  });
});
