import { describe, it, expect } from 'vitest';
import { fillTemplate } from '../src/strings.js';

describe('fillTemplate', () => {
  const mockContext = {
    CONSTANT1: 'Значение1',
    CONSTANT2: 'Значение2',
    CONSTANT3: Promise.resolve('Асинхронное значение'),
  };

  it('заменяет теги на значения констант', async () => {
    const template = 'Пример строки с включениями @CONSTANT1 и @CONSTANT2';
    const result = await fillTemplate(template, mockContext);
    expect(result).toEqual('Пример строки с включениями Значение1 и Значение2');
  });

  it('заменяет теги на значения констант (асинхронный случай)', async () => {
    const template = 'Пример строки с включениями @CONSTANT3';
    const result = await fillTemplate(template, mockContext);
    expect(result).toBe('Пример строки с включениями Асинхронное значение');
  });

  it('обрабатывает включения с регулярными выражениями', async () => {
    const template = 'Пример строки с включением @/CONSTANT3/';
    const result = await fillTemplate(template, mockContext);
    expect(result).toMatch(/^Пример строки с включением Асинхронное значение/);
  });

  it('сохраняет включения без подходящих значений', async () => {
    const template = 'Пример строки с включением @UNKNOWN';
    const result = await fillTemplate(template, mockContext);
    expect(result).toBe(template);
  });

  it('обрабатывает нестроковые значения', async () => {
    const template = 'Пример строки с включением @/CONSTANT[1,2]/ и числом @numberValue';
    mockContext.numberValue = 42;
    const result = await fillTemplate(template, mockContext);
    expect(result).toMatch(/^Пример строки с включением Значение.* и числом 42/);
  });

  it('подставляет значения в шаблон {{...}}', async () => {
    const template = "Hello, {{name}}! Today is {{day}}.";
    const context = { name: "Alice", day: "Monday" };
    const expected = "Hello, Alice! Today is Monday.";
    const result = await fillTemplate(template, context);
    expect(result).toEqual(expected);
  });

  it('корректно обрабатывает недостающие переменные', async () => {
    // FIXME: Тест может падать, если функция не поддерживает fallback
    const template = "Hello, {{name}}! Today is {{day}}.";
    const context = { day: "Wednesday" };
    const expected = "Hello, {{name}}! Today is Wednesday.";
    const result = await fillTemplate(template, context);
    expect(result).toEqual(expected);
  });

  it('обрабатывает шаблоны с фильтрацией по RegExp (pattern match)', async () => {
    // FIXME: Тест может падать, если реализация не ищет по ключам объекта
    const template = "My favorite fruit is {{fruitPattern:^[A-Za-z]*fruit$}}.";
    const context = {
      applefruit: "Apple",
      orangefruit: "Orange",
      bananafruit: "Banana"
    };
    const result = await fillTemplate(template, context);
    expect(result).toMatch(/^My favorite fruit is (Apple|Orange|Banana)\.$/);
  });

  it('обрабатывает числовые плейсхолдеры %d', async () => {
    // FIXME: Возможно требует кастомной поддержки в шаблонизаторе
    const template = "This is a numeric value: %d";
    const context = { value: 42 };
    const expected = "This is a numeric value: 42";
    const result = await fillTemplate(template, context);
    expect(result).toEqual(expected);
  });

  it('обрабатывает вложенные объекты в плейсхолдере', async () => {
    const template = "My car is {{car.brand}} {{car.model}}.";
    const context = {
      car: {
        brand: "Tesla",
        model: "Model S"
      }
    };
    const expected = "My car is Tesla Model S.";
    const result = await fillTemplate(template, context);
    expect(result).toEqual(expected);
  });
});
