import { describe, it, expect } from 'vitest';
import { getValueByPath } from '../src/objects.js';

describe('getValueByPath (async)', () => {
  const user = {
    name: 'Bob',
    links: {
      '000': 'link000',
      nested: { key: 'value' },
    },
    items: [{ id: 1 }, { id: 2 }],
    peers: {
      12289: { name: 'Alice' },
      10000: { name: 'Charlie' },
    },
    getPeer(id) {
      return this.peers[id];
    },
    async getPeerAsync(id) {
      return new Promise((resolve) => {
        setTimeout(() => resolve(this.peers[id]), 10);
      });
    },
    add(a, b) {
      return a + b;
    },
    async addAsync(a, b) {
      return new Promise((resolve) => {
        setTimeout(() => resolve(a + b), 10);
      });
    },
    asyncName: Promise.resolve('AsyncBob'),
    getAsyncName() {
      return Promise.resolve({ first: 'Async', last: 'Bob' });
    },
  };

  it('gets value with dot and bracket notation', async () => {
    expect(await getValueByPath(user, 'name')).toBe('Bob');
    expect(await getValueByPath(user, 'links.nested.key')).toBe('value');
    expect(await getValueByPath(user, 'links["000"]')).toBe('link000');
    expect(await getValueByPath(user, "links['000']")).toBe('link000');
  });

  it('gets value from array', async () => {
    expect(await getValueByPath(user, 'items[1].id')).toBe(2);
  });

  it('calls sync method with argument', async () => {
    expect(await getValueByPath(user, 'getPeer(12289).name')).toBe('Alice');
  });

  it('calls async method with argument', async () => {
    expect(await getValueByPath(user, 'getPeerAsync(10000).name')).toBe(
      'Charlie'
    );
  });

  it('calls method with multiple arguments (sync and async)', async () => {
    expect(await getValueByPath(user, 'add(2, 3)')).toBe(5);
    expect(await getValueByPath(user, 'addAsync(10, 15)')).toBe(25);
  });

  it('calls method with variable argument from context', async () => {
    const context = { peerId: 12289 };
    expect(await getValueByPath(user, 'getPeer(peerId).name', context)).toBe(
      'Alice'
    );
  });

  it('resolves async property', async () => {
    expect(await getValueByPath(user, 'asyncName')).toBe('AsyncBob');
  });

  it('resolves value returned by async method then accesses property', async () => {
    expect(await getValueByPath(user, 'getAsyncName().first')).toBe('Async');
    expect(await getValueByPath(user, 'getAsyncName().last')).toBe('Bob');
  });

  it('returns undefined for unknown property or method', async () => {
    expect(await getValueByPath(user, 'nonexistent')).toBeUndefined();
    expect(await getValueByPath(user, 'getPeer(99999).name')).toBeUndefined();
    expect(await getValueByPath(user, 'unknownMethod().foo')).toBeUndefined();
  });

  it('calls method with string literal and nested access', async () => {
    const user = {
      getLink(id) {
        return { id, href: `/link-${id}` };
      },
    };

    expect(await getValueByPath(user, 'getLink("l2").href')).toBe('/link-l2');
  });

  it('calls method with string literal and nested access', async () => {
    const data = {
      user: {
        name: 'Alice',
        links: [
          {
            id: 'l1',
            href: '/link1',
          },
          {
            id: 'l2',
            href: '/link2',
          },
        ],
        meta: {
          peer_0: {
            status: 'online',
          },
        },
        getLink: function (id) {
          return this.links.find((l) => l.id === id);
        },
      },
      numbers: [10, 20, 30],
    };

    const check = data.user.getLink('l2').href;
    const result = await getValueByPath(data, 'user.getLink("l2").href');

    expect(result).toBe(check);
  });

  it('calls method with string literal and nested async access', async () => {
    const data = {
      user: {
        name: 'Alice',
        links: [
          {
            id: 'l1',
            href: '/link1',
          },
          {
            id: 'l2',
            href: '/link2',
          },
        ],
        meta: {
          peer_0: {
            status: 'online',
          },
        },
        getLink: function (id) {
          return this.links.find((l) => l.id === id);
        },

        async getProfile() {
          return { nickname: 'Ali', level: 3 };
        },
      },
      numbers: [10, 20, 30],
    };

    const check = (await data.user.getProfile()).nickname;
    const result = await getValueByPath(data, 'user.getProfile().nickname');

    expect(result).toBe(check);
  });
});
