import { DateTime, Interval } from 'luxon';

export const TimeZones = {
  'MSK-1': 'Europe/Kaliningrad',
  'MSK': 'Europe/Moscow',
  'MSK+1': 'Europe/Astrakhan',
  'MSK+2': 'Asia/Yekaterinburg',
  'MSK+3': 'Asia/Omsk',
  'MSK+4': 'Asia/Krasnoyarsk',
  'MSK+5': 'Asia/Irkutsk',
  'MSK+6': 'Asia/Chita',
  'MSK+7': 'Asia/Vladivostok',
  'MSK+8': 'Asia/Magadan',
  'MSK+9': 'Asia/Kamchatka'
}

export function correctTimeZone(timeZone = 'MSK') {

  if (TimeZones[timeZone])
    return TimeZones[timeZone]
  else
    return timeZone;
}


export function toLuxonDateTime(dateTime, options = {}) {

  let result;

  options.setZone = options.setZone ?? true;

  if (dateTime instanceof DateTime)
    result = dateTime

  else if (dateTime instanceof Date)
    result = DateTime.fromJSDate(dateTime, { zone: options.zone })

  else if (typeof dateTime === 'string')
    if (typeof options.format === 'string')
      result = DateTime.fromFormat(dateTime, options.format, options)

    else
      result = DateTime.fromISO(dateTime, options)

  else if (typeof dateTime === 'number')
    if (options.type === 'seconds')
      result = DateTime.fromSeconds(dateTime, options)

    else
      result = DateTime.fromMillis(dateTime, options);

  return result;
}


export function createInterval(startDate, endDate, options = {}) {

  let
    result,
    from = toLuxonDateTime(startDate, options),
    to = toLuxonDateTime(endDate, options);

  if (from.isValid && to.isValid)
    result = Interval.fromDateTimes(from, to);

  return result;
}


function getWorkdaysInIntersection(startDate, endDate, interval, options = {}) {

  let
    result,
    intersection = interval.intersection(
      createInterval(startDate, endDate, options)
    );

  if (intersection) {
    result = calcWorkdays(intersection.start, intersection.end)
  } else
    result = 0;

  return result;
}


export function calcWorkdays(startDate, endDate, options = {}) {

  let result;
  let start = toLuxonDateTime(startDate).startOf('day');
  let end = toLuxonDateTime(endDate).endOf('day');

  if (options.zone) {
    let keepLocalTime = options.keepLocalTime ?? true
    start = start.setZone(options.zone, { keepLocalTime })
    end = end.setZone(options.zone, { keepLocalTime })
  }

  let interval = createInterval(start, end, options);
  let fullDuration = Math.ceil(interval.toDuration('days').as('days'));
  let startWeekday = interval.start?.weekday;
  let endWeekday = interval.end?.weekday;

  let head = 8 - startWeekday;
  let tail = (endWeekday === 7) ? 0 : endWeekday;
  let body = fullDuration - head - tail;

  let deltaHead = (head === 1) ? 1 : head - 2;
  let deltaTail = tail > 5 ? 5 : tail;

  result = (body / 7 * 5) + deltaHead + deltaTail;

  return result;
}


export function calcWorkdaysWithHolidays(startDate, endDate, options = {}, holidays) {

  let result;
  let start = toLuxonDateTime(startDate).startOf('day');
  let end = toLuxonDateTime(endDate).endOf('day');
  let workdays;
  let keepLocalTime = options.keepLocalTime ?? true;
  let zone;
  let from;
  let to;

  if (!options.keepZone) {
    zone = options.zone ?? start.zoneName;
    start = start.setZone(zone)
    end = end.setZone(zone)
  }

  workdays = calcWorkdays(start, end, options);
  result = workdays;

  if (typeof holidays === 'object' && holidays.from && holidays.to) {

    from = toLuxonDateTime(holidays.from);
    to = toLuxonDateTime(holidays.to);

    if (zone) {
      from = from.setZone(zone, { keepLocalTime })
      to = to.setZone(zone, { keepLocalTime })
    }

    let intersection = getWorkdaysInIntersection(
      start,
      end,
      createInterval(from, to, options),
      options
    );
    result = workdays - intersection;

  } else if (Array.isArray(holidays)) {
    for (let item of holidays) {

      from = toLuxonDateTime(item.from);
      to = toLuxonDateTime(item.to);

      if (zone) {
        from = from.setZone(zone, { keepLocalTime })
        to = to.setZone(zone, { keepLocalTime })
      }

      let intersection = getWorkdaysInIntersection(
        start,
        end,
        createInterval(from, to, options),
        options
      );
      result = result - intersection;
    }
  }

  return result;
}


export function addWorkdays(date, addDays, options = {}, holidays) {

  let result;
  let workdays;
  let startWorkdays;
  let goalWorkdays;
  let delta = addDays;
  let from = toLuxonDateTime(date, options);
  let to;

  from = from.startOf('day');
  to = from.endOf('day');

  startWorkdays = calcWorkdaysWithHolidays(from, to, options, holidays);
  goalWorkdays = startWorkdays + addDays;

  do {
    to = to.plus({ days: delta });
    workdays = calcWorkdaysWithHolidays(from, to, options, holidays);
    delta = goalWorkdays - workdays;
  } while (delta !== 0)

  result = to;

  return result;
}


export function calcSpeed(
  amount,
  startDate,
  endDate,
  options = {},
  holidays = {}
) {

  let
    result,
    workdays;

  if (typeof startDate === 'undefined') {
    startDate = DateTime.now();
    options = {};
  }

  workdays = calcWorkdaysWithHolidays(
    startDate,
    endDate ?? startDate,
    options,
    holidays
  );

  result = workdays === 0 ? 0 : (amount ?? 0) / workdays;

  return result;
}


export function isWorkTime(dateTime, workInterval = { from: '9:00', to: '18:00' }) {

  try {
    if (!dateTime)
      dateTime = DateTime.now();

    let
      zoneName = dateTime.zoneName,
      time = dateTime.toFormat('H:mm'),
      from = DateTime.fromFormat(workInterval.from, 'H:mm', { zone: zoneName }),
      to = DateTime.fromFormat(workInterval.to, 'H:mm', { zone: zoneName }),
      result;

    time = DateTime.fromFormat(time, 'H:mm', { zone: zoneName });

    let
      fromResult = (time >= from),
      toResult = (time < to);

    result = (fromResult && toResult);

    return result;

  } catch (error) {

    return false;
  }
}


export function isWorkday(date, holidays, options = {}) {

  let result = true;
  let weekday = toLuxonDateTime(date, options).weekday;

  if (weekday > 5) {
    //TODO: Учесть перенесенные выходные (часть суббот и воскресений может быть рабочими)
    result = false;

  } else if (typeof holidays === 'object' && holidays.from && holidays.to) {
    result = isDateTimeInInterval(date, holidays.from, holidays.to, options);

  } else if (Array.isArray(holidays)) {
    for (let item of holidays) {
      if (
        item.from
        && item.to
        && isDateTimeInInterval(date, item.from, item.to, options)
      ) {
        result = false;
        break;
      }
    }
  }

  return result;
}


export function isDateTimeInInterval(date, from, to, options = {}) {

  let result = false;

  let startDate = toLuxonDateTime(from, options);
  let endDate = toLuxonDateTime(to, options);
  let checkDate = toLuxonDateTime(date, options);

  if (options.fromStartOfDay) {
    if (options.zone) checkDate = checkDate.setZone(options.zone);

    startDate = startDate.setZone(checkDate.zoneName);
    endDate = endDate.setZone(checkDate.zoneName);

    startDate = startDate.startOf('day');
    endDate = endDate.endOf('day');
  }

  if (checkDate >= startDate && checkDate <= endDate) result = true;

  return result;
}