'use strict';

import fs from 'fs';
import path from 'path';
import { parse } from 'comment-json';
import { parse as yamlParse } from 'yaml';

import { isFileExists } from './files.js';

const configScope = [
  'config.yml',
  'config/config.yml',
  'config.jsonc',
  'config.json',
  'config/config.jsonc',
  'config/config.json',
];

export function log(string) {
  console.log(string);
}

export function error(string) {
  console.error(string);
}

export function delay(ms) {
  return new Promise((resolve) => {
    setTimeout(() => {
      resolve();
    }, ms);
  });
}

export function getCallerName(level) {
  try {
    const err = new Error();
    const stack = err.stack?.split('\n').slice(1) || [];

    // Если уровень явно указан — берём конкретную строку
    if (typeof level === 'number') {
      const line = stack[level];
      const match = line?.trim().match(/^at\s+([^\s(]+)\s?\(/);
      return match?.[1] || 'anonymous';
    }

    // Автоматический режим — ищем первое "вменяемое" имя
    for (const line of stack) {
      const match = line.trim().match(/^at\s+([^\s(]+)\s?\(/);
      const name = match?.[1];

      if (!name) continue;
      if (
        ['getCallerName', 'runTest', 'runSuite', 'runFiles'].includes(name) ||
        name.startsWith('file:')
      )
        continue;

      return name;
    }

    return 'anonymous';
  } catch {
    return 'anonymous';
  }
}

export async function getConfig(
  root,
  scope = configScope,
  removeComments = false
) {
  let rootPath = path.resolve(root ?? './'),
    configFileName,
    filePath,
    result;

  for (let item of scope) {
    filePath = path.join(rootPath, item);

    if (await isFileExists(filePath)) {
      configFileName = filePath;
      break;
    }
  }

  if (configFileName) {
    do {
      let config;

      switch (path.extname(configFileName)) {
        case '.yml':
          config = yamlParse(fs.readFileSync(filePath).toString(), {
            prettyErrors: true,
            logLevel: 'error',
          });
          break;

        case '.jsonc':
        case '.json':
          config = parse(
            fs.readFileSync(filePath).toString(),
            null,
            removeComments
          );
          break;

        default:
          throw new Error(
            `Unknown config file type: ${path.extname(configFileName)}`
          );
      }

      if (config?.redirect) filePath = config.redirect;
      else {
        config.fileName = configFileName;
        result = config;
      }
    } while (typeof result === 'undefined');

    return result;
  }
}

export function checkEnumValue(value, enumObject) {
  let result;

  if (Object.values(enumObject).includes(value)) {
    result = true;
  } else {
    result = false;
  }

  return result;
}

export function getConfigItem(configItem, defaultValue = '') {
  if (typeof configItem === 'string') {
    return configItem;
  }

  const { env } = configItem || {};
  return process.env[env] ?? defaultValue;
}
