/**
 * @module objects
 * @description This module contains routines for objects manipulation
 */
'use strict';

import { getRandomInt } from './numbers.js';

export function clone(object) {
  let objectString, result;

  try {
    if (object) {
      objectString = JSON.stringify(object);
      result = JSON.parse(objectString);
    }
  } catch (error) {}

  return result;
}

export function resolvePath(object, path, defaultValue) {
  return path.split('.').reduce(async (o, p) => {
    let result;

    if (o instanceof Promise) o = await o;

    result = o ? o[p] : defaultValue;

    return result;
  }, object);
}

export function objectToPath(obj, path = '') {
  let result = '';

  for (let key in obj) {
    if (typeof obj[key] === 'object' && obj[key] !== null) {
      result += objectToPath(obj[key], path + key + '.');
    } else result += path + key + '.';
  }

  return result.slice(0, -1);
}

export function setValueByPath(obj, path, value) {
  const parts = path.split('.');

  let current = obj;

  for (let i = 0; i < parts.length; i++) {
    if (typeof current !== 'undefined' && typeof parts[i] !== 'undefined')
      if (i === parts.length - 1) current[parts[i]] = value;
      else current = current[parts[i]];
  }

  return current;
}
export async function getValueByPath(obj, path, context = {}) {

  const tokenRegex =
  /\.?([a-zA-Z_$][\w$]*)\(([^)]*)\)|\[(?:(?:'([^']*)')|(?:"([^"]*)")|([^\]]+))\]|\.?([a-zA-Z_$][\w$]*)/g;


  let match;
  let current = obj;

  while ((match = tokenRegex.exec(path)) !== null) {
    const [
      ,
      methodName, // match[1]
      methodArgs, // match[2]
      bracketStr1, // match[3]
      bracketStr2, // match[4]
      bracketExpr, // match[5]
      dotKey, // match[6]
    ] = match;

    current = await current;

    if (methodName) {
      // Разбор аргументов
      /*
      const args = methodArgs
        .split(',')
        .map(arg => arg.trim())
        .map(arg => {
          if (/^['"]/.test(arg)) return arg.slice(1, -1);       // строка
          if (/^\d+$/.test(arg)) return Number(arg);            // число
          if (arg in context) return context[arg];              // переменная из контекста
          return undefined;
        });
      */
      const args = methodArgs
        .split(',')
        .map((arg) => arg.trim())
        .map((arg) => {
          if (
            (arg.startsWith('"') && arg.endsWith('"')) ||
            (arg.startsWith("'") && arg.endsWith("'"))
          ) {
            return arg.slice(1, -1);
          }

          if (/^\d+$/.test(arg)) return Number(arg);
          if (arg in context) return context[arg];
          return arg;
        });

      const fn = current?.[methodName];
      if (typeof fn === 'function') {
        current = fn.apply(current, args);
      } else {
        return undefined;
      }
    } else if (dotKey) {
      current = current?.[dotKey];
    } else if (bracketStr1 || bracketStr2) {
      const key = bracketStr1 || bracketStr2;
      current = current?.[key];
    } else if (bracketExpr) {
      const key = context?.[bracketExpr] ?? bracketExpr;
      current = current?.[key];
    }
  }

  return await current;
}


/**
 * This is a function to retrieve a random value from a list of key-value
 * pairs, where the keys satisfy a given regular expression
 * @param {string} pattern - A regular expression
 * @param {object} list - A list of key-value pairs
 * @example
 * let
 *   strings = {
 *     HELLO_1: 'Привет!',
 *     HELLO_2: 'Hi',
 *     HELLO_3: 'Салам!',
 *     HELLO_4: 'Guten Tag!',
 *     HELLO_5: 'Hola!'
 *   },
 *   pattern = 'HELLO_\\d',
 *   result;
 *
 * result = getRandomValue(pattern, strings);
 *
 * console.log(result); // value of random strings list's item (for example - 'Hi')
 * @returns {*} random value
 */
export function getRandomValue(pattern, list) {
  function shuffleArray(array) {
    for (var i = array.length - 1; i > 0; i--) {
      var j = Math.floor(Math.random() * (i + 1));
      var temp = array[i];
      array[i] = array[j];
      array[j] = temp;
    }
  }

  let result,
    matchedKeys = [];

  for (let key in list) if (key.match(pattern)) matchedKeys.push(list[key]);

  shuffleArray(matchedKeys);

  if (matchedKeys.length > 0)
    result = matchedKeys[getRandomInt(0, matchedKeys.length - 1)];

  return result;
}

export function isPrimitive(value) {
  let result;

  result = value !== Object(value);

  return result;
}

export function isPromise(obj) {
  let result;

  result = obj instanceof Promise;

  return result;
}
