/**
 * @module strings
 * @description This module contains routines for strings manipulation
 */
'use strict';

import { replaceAsync, filterAsync, mapAsync } from './async.js';

import { getRandomInt } from './numbers.js';

import { isPrimitive, resolvePath } from './objects.js';

const TAGS_PATTERN = /@(([A-Za-z_0-9]+)|(\/(.*)\/))/g;
const PARAMS_PATTERN = /.+\:\[[\s;,]*(.*)\]$/;
const TEMPLATE_PATTERN = /\{\{.*\}\}/;
const SEPARATOR_PATTERN = /\s*[@;,]\s*/;

/**
 * Fills placeholders in a template string using values from a context object.
 * Supports @TAGS, handlebars {{...}}, format-style %d/%s, regex-based inclusion.
 * @param {string} template - The template string.
 * @param {Object} context - Values to insert.
 * @returns {Promise<string>}
 */
export async function fillTemplate(template, context) {
  let result = template;
  let params;
  let parts;
  let placeholdersCount;

  // --- Обработка @CONSTANT и @/regexp/
  result = await replaceAsync(
    result,
    TAGS_PATTERN,
    async (match, _, tag, regexp, pattern) => {
      if (tag) {
        const value = await Promise.resolve(context[tag]);
        return value === undefined ? match : value;
      } else if (pattern) {
        const matchingKeys = await filterAsync(
          Object.keys(context),
          async (key) => {
            const val = await context[key];
            return isPrimitive(val) && new RegExp(pattern).test(key);
          }
        );

        if (matchingKeys.length > 0) {
          const randKey =
            matchingKeys[getRandomInt(0, matchingKeys.length - 1)];
          const value = await Promise.resolve(context[randKey]);
          return '' + value;
        }

        return match;
      } else {
        return match;
      }
    }
  );

  // --- Обработка printf-style (%d, %s, %j, etc.)
  if (/%[sdj%]/.test(result) && typeof context === 'object') {
    const args = Object.values(context);
    result = f(result, ...args);
  }

  // --- Обработка {{...}} и {{key:regex}}
  if (TEMPLATE_PATTERN.test(result)) {
    result = await replaceAsync(result, /\{\{(.*?)\}\}/g, async (_, expr) => {
      const [key, pattern] = expr.split(':').map((s) => s.trim());

      if (pattern) {
        const regex = new RegExp(pattern);
        const matches = Object.entries(context).filter(
          ([k, v]) => regex.test(k) && isPrimitive(v)
        );

        if (matches.length > 0) {
          const [, value] = matches[getRandomInt(0, matches.length - 1)];
          return await Promise.resolve(value);
        }

        return `{{${expr}}}`;
      }

      const value = await resolvePath(context, key);
      return value !== undefined ? value : `{{${key}}}`;
    });
  }

  // --- Обработка шаблонов с параметрами :[a, b, c] → форматирование через f()
  parts = template.match(PARAMS_PATTERN);
  if (Array.isArray(parts) && parts[1]) {
    params = parts[1].split(SEPARATOR_PATTERN);
    placeholdersCount = countPlaceholders(result);
    if (Array.isArray(params) && params.length > 0 && placeholdersCount > 0) {
      const args = await mapAsync(params.splice(0, placeholdersCount), (item) =>
        resolvePath(context, item)
      );
      result = f(result, ...args);
    }
  }

  return result;
}

/**
 * This is a function that checks if a text string matches a specified pattern,
 * which can be provided either as a plain string or a regular expression
 * @param {string} text - Text string for pattern matching
 * @param {string} pattern - Pattern for matching
 * @param {string} [isRegExp] - Flag indicating that the pattern is a regular
 *     expression
 * @param {string} [flags] - String with a set of flags for matching text
 *     against a regular expression<br>
 *     <p><b>Flags for regular expressions:</b></p>
 *     <ul>
 *     <li>'i' - Case-insensitive matching</li>
 *     <li>'g' - Global matching (finds all matches, not just the first one)</li>
 *     <li>'m' - Multiline mode</li>
 *     <li>'s' - Dotall mode, where the dot . matches any character including
 *              newlines</li>
 *     <li>'u' - Enables full Unicode support, allowing correct handling of
 *              surrogate pairs</li>
 *     <li>'y' - Sticky mode for searching at a particular position in the text</li>
 *     </ul>
 * @example
 * let
 *   text = 'test',
 *   pattern = 'Te.*',
 *   flags = 'i',
 *   result;
 *
 * result = isTextMatch(text, pattern, true, flags);
 *
 * console.log(result); // true
 * @returns {boolean} True if the pattern matches. Else - false
 */
export function isTextMatch(text, pattern, isRegExp, flags) {
  let result = false;

  if (isRegExp) {
    text = text ?? '';
    let re = new RegExp(pattern, flags);
    result = re.test(text);
  } else result = text === pattern;

  return result;
}

/**
 * Generates a random string of specified length and character set.
 * @param {number} len - The length of the random string.
 * @param {string} [charSet] - The character set to use. Defaults to alphanumeric characters.
 * @returns {string} The generated random string.
 */
export function getRandomString(len, charSet) {
  let randomString = '';

  charSet =
    charSet || 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';

  for (var i = 0; i < len; i++) {
    var randomPoz = Math.floor(Math.random() * charSet.length);
    randomString += charSet.substring(randomPoz, randomPoz + 1);
  }

  return randomString;
}

/**
 * Counts the number of placeholders in a template string.
 * @param {string} template - The template string to count placeholders in.
 * @returns {number} The number of placeholders found in the template string.
 */
export function countPlaceholders(template) {
  let result = 0;

  if (typeof template === 'string') {
    let matches = template.match(/%[sdijoOf]/g);
    if (Array.isArray(matches)) result = matches.length;
  }

  return result;
}

/**
 * Checks if a text string matches any of the regular expressions in the given array.
 * @param {string} str - The text string to match.
 * @param {RegExp[]} regexArray - An array of regular expressions to match against.
 * @returns {boolean} `true` if the text matches any of the regular expressions, otherwise `false`.
 */
export function matchesAnyRegExp(str, regexArray) {
  return regexArray.some((regex) => regex.test(str));
}

/**
 * Formats a string using placeholders similar to `util.format`
 *
 * It just ignored the extra arguments that do not correspond to
 * format specifiers
 *
 * Supports the following format specifiers:
 * - `%s` for string conversion
 * - `%d` for integer conversion
 * - `%j` for JSON string conversion
 * - `%%` for literal percent signs
 *
 * @param {string} template - The string template containing format specifiers.
 * @param {...*} args - Values to replace the format specifiers in the template.
 * @returns {string} The formatted string with the placeholders replaced by corresponding values.
 *
 * @example
 * // Returns: "My name is Vladislav, I am 30 years old."
 * console.log(format('My name is %s, I am %d years old.', 'Vladislav', 30));
 *
 * @example
 * // Returns: "JSON object: {\"name\":\"Vladislav\",\"age\":30}"
 * console.log(format('JSON object: %j', { name: 'Vladislav', age: 30 }));
 *
 * @example
 * // Returns: "Percent: %"
 * console.log(format('Percent: %%'));
 *
 * @example
 * // Returns: "Values: text, 42"
 * console.log(format('Values: %s, %d', 'text', 42, 'extra'));
 */
export function f(template, ...args) {
  let index = 0;

  return template.replace(/%([sdj%])/g, (match, specifier) => {
    const value = args[index++];

    switch (specifier) {
      case 's':
        return String(value);
      case 'd':
        return Number.isFinite(value) ? Math.floor(Number(value)) : 'NaN';
      case 'j':
        try {
          return JSON.stringify(value);
        } catch {
          return '[Circular]';
        }
      case '%':
        return '%';
      default:
        return match;
    }
  });
}
